import log from './logger';

const ROUTE_IMPORT_VALUE = 'vtex.renderjs/route.js';

function createFileMessage(message, path, state) {
  const filename = state.file.opts.filename;
  const { loc } = path.node;
  return `${ message } on ${ filename } (${ loc.start.line }:${ loc.start.column })`;
}

export default function ({ types: t }) {
  let routeFnIdentifier;
  return {
    visitor: {
      ImportDeclaration(path) {
        const isRouteImport = path.node.source.value === ROUTE_IMPORT_VALUE;
        const isDefaultImport = t.isImportDefaultSpecifier(path.node.specifiers[0]);

        if (isRouteImport && isDefaultImport) {
          routeFnIdentifier = path.node.specifiers[0].local.name;
        }
      },
      CallExpression(path, state) {
        const routeBinding = path.scope.bindings[routeFnIdentifier];
        const isRouteCallee = path.node.callee.name === routeFnIdentifier;
        const isModule = routeBinding ? routeBinding.kind === 'module' : null;

        if (isRouteCallee && isModule) {
          const [nameArg, pathArg] = path.node.arguments;
          const isNameString = t.isStringLiteral(nameArg);
          const isPathString = t.isStringLiteral(pathArg);

          if (path.node.arguments.length > 2) {
            log.warn(createFileMessage(`Too many arguments on ${ routeFnIdentifier } function`, path, state));
          }

          if (!isNameString || !isPathString) {
            log.error(createFileMessage(`Arguments to the ${ routeFnIdentifier } function must be string literals`, path, state));
            return;
          }

          state.file.metadata.vtexRender = {
            route: {
              name: nameArg.value,
              path: pathArg.value
            }
          };
        }
      }
    }
  };
}